package org.kapelle.hardcore_revive;

import org.bukkit.GameMode;
import org.bukkit.Location;
import org.bukkit.Material;
import org.bukkit.OfflinePlayer;
import org.bukkit.block.Block;
import org.bukkit.block.Skull;
import org.bukkit.entity.Player;
import org.bukkit.event.block.BlockPlaceEvent;
import org.bukkit.plugin.java.JavaPlugin;
import org.bukkit.util.BlockVector;
import org.bukkit.util.Vector;
import org.kapelle.multiblock.MultiblockListener;
import org.kapelle.multiblock.MultiblockStructure;
import org.kapelle.multiblock.block_component.IBlockComponent;
import org.kapelle.multiblock.block_component.MultipleBlocks;
import org.kapelle.multiblock.block_component.SingleBlock;

import java.util.Arrays;
import java.util.HashSet;

public class RevivePlayer{
    private final JavaPlugin plugin;

    public RevivePlayer(JavaPlugin plugin){
        this.plugin = plugin;

        new MultiblockListener(plugin, getMultiblockStructure(), new BlockVector(1, 2, 1)) {
            @Override
            protected void callback(Location coreBlockLocation) {
                onMultiblockValid(coreBlockLocation);
            }
        };
    }

    private MultiblockStructure getMultiblockStructure() {
        IBlockComponent gold = new SingleBlock(Material.GOLD_BLOCK);
        IBlockComponent soulSand = new MultipleBlocks(new HashSet<Material>() {{
            add(Material.SOUL_SAND);
            add(Material.SOUL_SOIL);
        }});
        IBlockComponent head = new SingleBlock(Material.PLAYER_HEAD);
        IBlockComponent air = new SingleBlock(Material.AIR);
        IBlockComponent obsidian = new SingleBlock(Material.CRYING_OBSIDIAN);
        IBlockComponent dia = new SingleBlock(Material.DIAMOND_BLOCK);

        return new MultiblockStructure(
                Arrays.asList(
                        // Bottom layer
                        Arrays.asList(
                                Arrays.asList(gold,     obsidian,   gold),
                                Arrays.asList(obsidian, dia,        obsidian),
                                Arrays.asList(gold,     obsidian,   gold)
                        ),

                        // Mid layer
                        Arrays.asList(
                                Arrays.asList(air, air, air),
                                Arrays.asList(air, soulSand, air),
                                Arrays.asList(air, air, air)

                        ),

                        // Top layer
                        Arrays.asList(
                                Arrays.asList(air, air, air),
                                Arrays.asList(air, head, air),
                                Arrays.asList(air, air, air)
                        )

                ));
    }

    private void onMultiblockValid(Location coreBlockLocation){
        OfflinePlayer spirit = getPlayerFromHead(coreBlockLocation.getBlock());

        if (spirit.isOnline() /* && spirit.getPlayer().getGameMode() == GameMode.SPECTATOR */){
            revivePlayer(spirit.getPlayer(),coreBlockLocation);
        }else{
            // Player is not online or not dead
            plugin.getServer().broadcastMessage("Player not ded");
            // TODO
        }
    }

    private OfflinePlayer getPlayerFromHead(Block head){
        Skull skull = (Skull) (head.getState());
        return skull.getOwningPlayer();
    }

    private void revivePlayer(Player player, Location headLocation){
        // Break blocks
        headLocation.getBlock().setType(Material.AIR);
        headLocation.add(new Vector(0,-1,0)).getBlock().setType(Material.AIR);
        headLocation.add(new Vector(0,-1,0)).getBlock().setType(Material.AIR);

        // Teleport player
        player.teleport(headLocation);

        // Set player gamemode
        player.setGameMode(GameMode.SURVIVAL);
    }
}
