package org.kapelle.hardcore_revive;

import org.bukkit.Location;
import org.bukkit.Material;
import org.bukkit.Particle;
import org.bukkit.block.Block;
import org.bukkit.block.Sign;
import org.bukkit.entity.Player;
import org.bukkit.inventory.ItemStack;
import org.bukkit.inventory.meta.SkullMeta;
import org.bukkit.plugin.java.JavaPlugin;
import org.bukkit.util.BlockVector;
import org.bukkit.util.Vector;
import org.kapelle.multiblock.MultiblockListener;
import org.kapelle.multiblock.MultiblockStructure;
import org.kapelle.multiblock.block_component.IBlockComponent;
import org.kapelle.multiblock.block_component.MultipleBlocks;
import org.kapelle.multiblock.block_component.SingleBlock;

import java.util.Arrays;
import java.util.HashSet;

public class SummonHead{
    private final JavaPlugin plugin;

    public SummonHead(JavaPlugin plugin){
        this.plugin = plugin;

        new MultiblockListener(plugin, getMultiblockStructure(), new BlockVector(0, 0, 0)) {
            @Override
            protected void callback(Location coreBlockLocation) {
                summonHead(coreBlockLocation);
            }
        };
    }

    private MultiblockStructure getMultiblockStructure(){
        IBlockComponent sign = new MultipleBlocks(new HashSet<Material>(){{
            add(Material.SPRUCE_SIGN);
            add(Material.ACACIA_SIGN);
            add(Material.CRIMSON_SIGN);
            add(Material.JUNGLE_SIGN);
            add(Material.OAK_SIGN);
            add(Material.WARPED_SIGN);
        }});

        IBlockComponent hayBale = new SingleBlock(Material.HAY_BLOCK);

        IBlockComponent pumpkin = new MultipleBlocks(new HashSet<Material>(){{
            add(Material.JACK_O_LANTERN);
            add(Material.CARVED_PUMPKIN);
        }});

        return new MultiblockStructure(Arrays.asList(
                // Bottom layer
                Arrays.asList(
                        Arrays.asList(hayBale)
                ),

                // Mid Layer
                Arrays.asList(
                        Arrays.asList(pumpkin)
                ),

                // Top layer
                Arrays.asList(
                        Arrays.asList(sign)
                )
        ));
    }

    /**
     * Perform the head summoning ritual.
     * @param coreBlockLocation The location of the core block of the multiblock structure. In this case
     *                          the soul campfire.
     */
    private void summonHead(Location coreBlockLocation){
        Location signBlock = coreBlockLocation.clone().add(new Vector(0,2,0));
        Location pumpkin = coreBlockLocation.clone().add(new Vector(0,1,0));

        String spiritName = getSignText((Sign) signBlock.getBlock().getState());
        Player spirit = plugin.getServer().getPlayerExact(spiritName);

        if (spirit == null) {
            // TODO: inform player about spirit not readchable (must be online)
            return;
        }

        // Remove sign
        signBlock.getBlock().setType(Material.AIR);
        pumpkin.getBlock().setType(Material.AIR);

        // Spawn particle at sign
        pumpkin.getWorld().spawnParticle(Particle.SPELL_WITCH,pumpkin,20);

        // Prepare head to be spawned
        ItemStack head = new ItemStack(Material.PLAYER_HEAD);
        SkullMeta headMeta = (SkullMeta) head.getItemMeta();
        assert headMeta != null;
        headMeta.setOwningPlayer(spirit);
        head.setItemMeta(headMeta);

        // Spawn head at the position of the sign
        pumpkin.getWorld().dropItem(pumpkin,head);
    }

    /**
     * Joins all lines from a sign. That way usernames that are too long for a single line can fit.
     * @param sign
     * @return
     */
    private String getSignText(Sign sign){
        return String.join("",sign.getLines());
    }

}
